/* -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
 * auto exposure
 * @brief Io␳tB^
 *
 * @param EV+ Io␳l
 * -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+-- */
#extension GL_ARB_texture_rectangle: enable

// Screen Efects󂯎
uniform sampler2DRect se_rgbaTex;    // RGBA
uniform vec2 se_screenSize;

//: name="Auto Exposure" type="bool" value=true
uniform bool ae;

//: name="EV+" type="slider" range=(-2.0, +2.0) value=0.0
uniform float ev;

// jARGBPx
float linearRGBtoY(const vec3 v) {
  return 0.299 * v.r + 0.587 * v.g + 0.114 * v.b;
}

/* -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
 * main
 * -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+-- */
void main(void) {
    vec2 uvPos = gl_FragCoord.xy;
    vec4 rgba = texture2DRect(se_rgbaTex, uvPos).xyzw;
    
    float w = se_screenSize.x;
    float h = se_screenSize.y;
    
    vec2 photometryPoints[9] = {
        vec2(0.25*w, 0.25*h), vec2(0.50*w, 0.25*h), vec2(0.75*w, 0.25*h),
        vec2(0.25*w, 0.50*h), vec2(0.50*w, 0.50*h), vec2(0.75*w, 0.50*h),
        vec2(0.25*w, 0.75*h), vec2(0.50*w, 0.75*h), vec2(0.75*w, 0.75*h)
    };
    
    float intensity = 0.0;
    if (ae) {
        float y;
        for (int i = 0; i < 9; ++i) {
            y = linearRGBtoY(texture2DRect(se_rgbaTex, photometryPoints[i]).rgb);
            y = max(y, 0.0);
            y = min(y, 1.0);
            intensity += y;
        }
        intensity /= 9.0;
    } else {
        intensity = 1.0;
    }
    
    vec3 rgb = (0.2 * pow(2.0, ev)/intensity) * rgba.rgb;
    gl_FragColor = vec4(rgb.x, rgb.y, rgb.z, rgba.w);
    
    return;
}
