/* -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
 * diffraction circular
 * @brief ܌tB^i~`j
 * 
 * @param F Fl
 *
 * ܂ɂu[ʂV~[g܂D
 * i`͉~`łD
 *
 * Reference ::
 * Kakimoto M., Matsuoka K., Nishita T., Naemura T., Harashima H.
 * "Glare generation based on wave optics".
 * Computer Graphics Forum, Vol. 24, No. 2. pp.185--193, 2005-7.
 * -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+-- */
#extension GL_ARB_texture_rectangle: enable

// Screen Efects󂯎
uniform sampler2DRect se_rgbaTex;    // RGBA
uniform vec2 se_screenSize;
uniform float se_cameraZoom;
uniform float se_cameraFocus; 

/* -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
 * main
 * -+----1----+----2----+----3----+----4----+----5----+----6----+----7----+-- */
void main(void) {
    vec2 uvPos = gl_FragCoord.xy;
    vec2 pos;
    vec4 rgba = texture2DRect(se_rgbaTex, uvPos).xyzw;
    vec3 rgb = vec3(0.0, 0.0, 0.0);
    
    // Fl
    float FValue = 1.0e+9;
    if (se_cameraFocus != 0.0) {
        FValue = 2.0 * 36.0 * (0.001 * se_cameraZoom) * (0.001 * se_cameraZoom)
                 / se_cameraFocus;
    }
    if (FValue > 512.0) {
        FValue = 512.0;
    }
    
    float pixWidth = max(se_screenSize.x, se_screenSize.y);
    float dotPitch = 0.036 / pixWidth;
    float airyRadius = 1.2196 * 435.8e-9 * FValue;
    float airyZeroPoint = 4.61089134947891;    
    float scalingFactor = airyRadius / airyZeroPoint / dotPitch;    // J[lԂ1pxXN[ŉpxɑ邩
    
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  7.63971e-07,  6.28339e-07,  1.10876e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(   8.7002e-07,  8.15698e-07,  9.06498e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  8.91719e-07,  7.11482e-07,  6.60279e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  9.60741e-07,  6.74531e-07,  5.81338e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.09434e-06,  7.43447e-07,  9.06116e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  9.87914e-07,  1.04389e-06,  8.36285e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.57149e-06,  8.17286e-07,  9.30124e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.33244e-06,   1.0973e-06,   1.0635e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.42261e-06,  1.18187e-06,  1.04914e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.80027e-06,  1.36306e-06,  1.19696e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.99898e-06,  1.33267e-06,  1.69378e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.55081e-06,  1.65345e-06,  1.02271e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  2.57248e-06,  1.90775e-06,  1.74478e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.83726e-06,  1.22512e-06,  2.32383e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  2.33181e-06,  1.52469e-06,   1.0061e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  2.66134e-06,  1.84874e-06,  2.41824e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  2.38551e-06,  1.61283e-06,  1.12969e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.85946e-06,  1.12982e-06,  2.19143e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  2.50798e-06,  2.03757e-06,  1.82014e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(   1.6209e-06,  1.65428e-06,  1.04821e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  2.07504e-06,  1.31772e-06,  1.71382e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.80492e-06,  1.29986e-06,  1.18565e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.44602e-06,  1.25438e-06,  1.03697e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(   1.2707e-06,  9.96036e-07,  1.02585e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(   1.5159e-06,  8.61439e-07,  8.89508e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.10812e-06,  1.06384e-06,  8.77189e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  1.04383e-06,  7.31031e-07,  9.28723e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  9.87393e-07,  7.29633e-07,  6.21449e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  9.14486e-07,  6.86449e-07,   6.1909e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  8.74537e-07,  8.20617e-07,  9.59176e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-15.0) * scalingFactor;
    rgb += vec3(  7.11943e-07,  6.29423e-07,  1.10898e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  8.44517e-07,  8.20779e-07,  9.32948e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  9.76279e-07,  8.32531e-07,  6.18186e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(    1.149e-06,   8.8657e-07,  1.04575e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.19145e-06,   9.6326e-07,  6.77295e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.21936e-06,  7.56945e-07,  6.04294e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.21937e-06,  9.49723e-07,  8.31676e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.67924e-06,   1.1525e-06,   9.1051e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.81904e-06,  1.40144e-06,  1.08907e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.68213e-06,  1.30652e-06,  1.01267e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.09058e-06,  1.61687e-06,  1.24394e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.34326e-06,  1.77989e-06,  1.53739e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.10395e-06,  1.92251e-06,  1.06831e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  3.03678e-06,  2.07501e-06,  1.62948e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.48584e-06,  1.81882e-06,  2.09968e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.65215e-06,  1.86035e-06,  9.72767e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.76764e-06,  2.39261e-06,  2.13241e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.66186e-06,  1.93555e-06,  1.08176e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.53136e-06,  1.68506e-06,  1.99669e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.96303e-06,  2.23023e-06,  1.69879e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.12898e-06,  1.94401e-06,  1.05115e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.44252e-06,  1.74322e-06,  1.59436e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  2.10587e-06,  1.55521e-06,  1.23282e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.71138e-06,  1.41435e-06,  9.79249e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.73669e-06,  1.33607e-06,  1.06164e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.76644e-06,  1.12131e-06,  9.14228e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.16683e-06,  9.83076e-07,  8.49879e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(   1.2727e-06,  8.27742e-07,  6.05801e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(   1.2023e-06,  9.42006e-07,  7.04645e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.10999e-06,  8.70237e-07,  9.90844e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  1.02846e-06,  8.46171e-07,   6.7886e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-14.0) * scalingFactor;
    rgb += vec3(  8.77203e-07,  8.24421e-07,  9.59586e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  8.95914e-07,  6.94761e-07,  5.08701e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.12303e-06,  8.79044e-07,  1.04965e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.11694e-06,  1.04293e-06,   9.7241e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.52074e-06,  1.11843e-06,  9.72204e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.36405e-06,  1.13481e-06,  9.70255e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.63644e-06,  1.12639e-06,  7.27643e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.81275e-06,  1.30169e-06,  1.22239e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.12621e-06,  1.68883e-06,  1.18514e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.23801e-06,  1.57678e-06,  1.29669e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.39013e-06,  1.99538e-06,  1.53015e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.92838e-06,  2.20233e-06,  1.77796e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.74449e-06,  2.32075e-06,  1.38021e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  3.47393e-06,  2.60822e-06,  1.83556e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  3.25689e-06,  2.48413e-06,  2.79344e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(    3.233e-06,  2.67287e-06,  1.44144e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  3.63696e-06,  3.19215e-06,  2.87095e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  3.28125e-06,  2.78842e-06,  1.57177e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  3.21817e-06,  2.33133e-06,  2.70572e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  3.46567e-06,  2.75278e-06,  1.91486e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.74651e-06,  2.36484e-06,  1.34725e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.98819e-06,  2.14982e-06,  1.83898e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.47823e-06,  2.00867e-06,  1.53621e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.19543e-06,  1.60584e-06,  1.23093e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  2.16556e-06,  1.73785e-06,  1.24259e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.90862e-06,  1.28623e-06,  1.17271e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.59437e-06,  1.15199e-06,  7.30558e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.38513e-06,  1.09527e-06,  1.01793e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.50101e-06,   1.1099e-06,  9.39594e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.14464e-06,  1.07025e-06,  9.72265e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  1.11047e-06,  8.72815e-07,  9.89107e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-13.0) * scalingFactor;
    rgb += vec3(  9.13631e-07,  6.84462e-07,  6.19964e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  9.74586e-07,  6.53604e-07,  5.75615e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.15246e-06,  9.35839e-07,  6.90054e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.47703e-06,  1.13291e-06,  9.30242e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.91469e-06,  1.25416e-06,  1.11662e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.92662e-06,   1.5015e-06,  1.15433e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  2.11564e-06,  1.53119e-06,   1.0868e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  2.58273e-06,  1.52665e-06,   9.7419e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  2.57532e-06,  1.82419e-06,  1.28473e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  3.17715e-06,   2.1849e-06,  1.72245e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  3.24656e-06,  2.40018e-06,  1.54889e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  4.15917e-06,  2.83325e-06,  1.87671e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  3.82158e-06,  3.05569e-06,  1.85792e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(   5.1764e-06,  3.27171e-06,   2.1659e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  4.59661e-06,  3.23022e-06,  2.97558e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  5.35221e-06,  3.42367e-06,   1.7773e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(   5.2125e-06,  3.53398e-06,  2.92697e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  5.39174e-06,  3.47892e-06,  1.88059e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  4.63168e-06,  3.11008e-06,   2.9098e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  5.24012e-06,  3.41842e-06,  2.24317e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  3.78397e-06,  3.09515e-06,    1.761e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  4.12228e-06,  2.76904e-06,  1.96437e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  3.38214e-06,   2.5069e-06,  1.58522e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(   3.1581e-06,   2.0893e-06,  1.60677e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  2.68797e-06,   1.8614e-06,  1.38213e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  2.54712e-06,  1.62598e-06,  1.00404e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  2.18024e-06,  1.48354e-06,  1.05556e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.87511e-06,  1.49112e-06,  1.10582e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.99308e-06,  1.30847e-06,  1.13719e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.50151e-06,  1.11407e-06,  9.42321e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  1.20471e-06,  9.37562e-07,  7.03751e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-12.0) * scalingFactor;
    rgb += vec3(  9.86449e-07,  7.31483e-07,  6.14771e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(   1.0009e-06,   7.6944e-07,  8.15269e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  1.25754e-06,  7.91948e-07,  6.95599e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(   1.4049e-06,  1.02992e-06,  9.54904e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  1.85428e-06,  1.52297e-06,  1.11211e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  2.15008e-06,  1.71599e-06,  1.42875e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  2.22884e-06,  1.97254e-06,  1.37772e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  3.01305e-06,  1.83411e-06,  1.52021e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  2.88519e-06,  2.30435e-06,  1.41537e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  3.39317e-06,   2.8015e-06,  2.21528e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  3.79272e-06,  2.98072e-06,  2.10415e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  4.68705e-06,  3.60309e-06,  2.31178e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  4.80077e-06,  3.94161e-06,  2.92412e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  5.68554e-06,  4.13888e-06,  2.85236e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  6.05997e-06,   4.5467e-06,  3.98468e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  5.54629e-06,  4.40161e-06,  2.75015e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  6.18943e-06,  4.81109e-06,  4.25187e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  5.64499e-06,  4.47973e-06,  2.85771e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  6.03139e-06,  4.44848e-06,  3.97011e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  5.78535e-06,  4.23051e-06,  2.92683e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  4.77219e-06,  3.97508e-06,   2.7551e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  4.69419e-06,  3.61857e-06,  2.41366e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(   3.8633e-06,  3.09155e-06,  2.06906e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  3.51467e-06,  2.76767e-06,  2.29643e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  2.94741e-06,  2.33727e-06,  1.42422e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  2.96385e-06,  1.78486e-06,   1.4876e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  2.27786e-06,  1.97601e-06,   1.3862e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  2.22416e-06,  1.81181e-06,  1.46196e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  1.87037e-06,  1.49161e-06,   1.1105e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  1.38554e-06,  1.09314e-06,   1.0156e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  1.27007e-06,  8.31988e-07,  6.04708e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-11.0) * scalingFactor;
    rgb += vec3(  1.04735e-06,  7.24426e-07,  9.24238e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  1.10781e-06,  1.02101e-06,  8.47031e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  1.13464e-06,  1.03092e-06,  8.14217e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  1.53383e-06,  1.15071e-06,  7.61984e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  2.17866e-06,  1.37189e-06,  9.90508e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  2.17786e-06,   1.9846e-06,  1.42198e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  2.97283e-06,  2.28362e-06,  1.60267e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  3.41107e-06,  2.48356e-06,  1.93884e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  3.80203e-06,  2.93191e-06,  1.88808e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  4.10966e-06,  3.32068e-06,  2.13175e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  4.78445e-06,  3.90136e-06,  2.63712e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  6.18046e-06,  4.48639e-06,  2.95231e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  6.23199e-06,  5.26417e-06,   3.6177e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  7.24997e-06,   5.5016e-06,  3.72539e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  8.00304e-06,  6.43113e-06,  4.70244e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  8.56097e-06,  6.58105e-06,  3.90765e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  9.23198e-06,  7.04493e-06,  4.82246e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  8.73582e-06,  6.75314e-06,  3.98352e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  7.87142e-06,  6.24944e-06,  4.72237e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  7.48493e-06,  5.62838e-06,  3.76842e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  6.26068e-06,  5.26508e-06,  3.49094e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  6.27098e-06,  4.63174e-06,  2.96775e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  4.82628e-06,  3.87623e-06,  2.70043e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  4.16369e-06,  3.45962e-06,   2.2258e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  3.80387e-06,  2.92813e-06,  1.86776e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  3.53492e-06,  2.50014e-06,  1.93127e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  2.95739e-06,   2.2792e-06,  1.65024e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  2.27472e-06,  1.97791e-06,  1.38681e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  2.17976e-06,  1.48412e-06,  1.05508e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  1.59301e-06,  1.15273e-06,  7.32283e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  1.17181e-06,  9.78904e-07,  8.46304e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-10.0) * scalingFactor;
    rgb += vec3(  1.10428e-06,  1.06529e-06,  8.79801e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.48006e-06,  8.59736e-07,  8.75961e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.74868e-06,  1.03246e-06,  8.52068e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.88717e-06,  1.29333e-06,  1.14207e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  2.45666e-06,   1.5893e-06,  1.06231e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  2.91966e-06,  1.78176e-06,  1.41147e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  3.38219e-06,  2.40718e-06,  1.90123e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  4.76627e-06,  2.80189e-06,  2.49129e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  5.10334e-06,  3.34668e-06,  2.74125e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(     5.98e-06,  4.00652e-06,  3.23393e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  6.94113e-06,   4.6553e-06,  3.38175e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  8.66448e-06,  5.34841e-06,  4.33086e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  9.53498e-06,  6.26629e-06,  5.08188e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.07114e-05,   7.1512e-06,  5.58758e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.27292e-05,  8.17929e-06,  6.66202e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.17623e-05,  8.27943e-06,  6.08526e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.20402e-05,  8.52633e-06,   7.6853e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.17606e-05,  8.43559e-06,  6.18897e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(   1.2777e-05,  8.06957e-06,  6.73249e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.09642e-05,   7.2012e-06,  5.56535e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(   9.5319e-06,  6.35065e-06,  5.14529e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  8.89506e-06,  5.52839e-06,  4.36746e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  7.01045e-06,  4.67942e-06,  3.42891e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  5.98203e-06,  3.99625e-06,  3.19405e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(   5.2704e-06,  3.44408e-06,  2.74676e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  4.71901e-06,  2.80989e-06,  2.51182e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  3.52685e-06,  2.49854e-06,  1.93256e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  2.96786e-06,  1.78385e-06,  1.48762e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  2.54773e-06,  1.62951e-06,  1.00109e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.91085e-06,  1.28459e-06,   1.1677e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.76215e-06,  1.12393e-06,  9.21615e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-9.0) * scalingFactor;
    rgb += vec3(  1.52199e-06,  8.67946e-07,  8.99362e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(   1.2783e-06,  9.96314e-07,  1.00897e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.69919e-06,  1.31968e-06,   1.0219e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  2.14241e-06,  1.69688e-06,  1.25287e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  2.65573e-06,  1.91534e-06,  1.31598e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  2.87663e-06,  2.20818e-06,  1.43074e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  3.69796e-06,  2.92037e-06,  1.84248e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  5.09306e-06,  3.34038e-06,  2.68837e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  5.61392e-06,   4.6471e-06,  3.25261e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  7.02037e-06,  5.44229e-06,  3.90317e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  8.31158e-06,  6.75305e-06,  4.46036e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(   1.0686e-05,  7.57629e-06,  5.64742e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.17362e-05,  9.37427e-06,  6.13681e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(    1.301e-05,  1.05391e-05,  7.83119e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.66586e-05,  1.26909e-05,  8.76391e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.69785e-05,  1.34176e-05,  9.08808e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.91612e-05,  1.22625e-05,  9.42667e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.73543e-05,  1.35148e-05,  9.16871e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.67143e-05,  1.26549e-05,  8.86331e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.33144e-05,  1.06692e-05,  7.70979e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.19139e-05,  9.57999e-06,  6.36229e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.08162e-05,   7.7003e-06,  5.65556e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  8.42005e-06,   6.8139e-06,  4.47296e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  7.16966e-06,  5.54402e-06,  3.95285e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  5.59026e-06,  4.66377e-06,   3.2779e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  5.26251e-06,  3.43867e-06,  2.74916e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(   3.8072e-06,  2.92624e-06,  1.86749e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  2.94727e-06,  2.34063e-06,  1.42097e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  2.69111e-06,  1.85803e-06,  1.38201e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  2.16524e-06,  1.74352e-06,  1.25422e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.74946e-06,  1.34722e-06,  1.06212e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-8.0) * scalingFactor;
    rgb += vec3(  1.27689e-06,  9.96981e-07,  1.01856e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.40777e-06,  1.23641e-06,  9.98072e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.65155e-06,  1.40776e-06,  9.85994e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  2.09252e-06,  1.60643e-06,  1.20033e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  3.15143e-06,  2.02952e-06,  1.61096e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  3.44438e-06,  2.75958e-06,   2.2549e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(   4.1225e-06,  3.34463e-06,  2.20734e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  5.78352e-06,  3.99789e-06,  3.08312e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  7.06611e-06,  5.34401e-06,  3.89183e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  8.26806e-06,  7.13499e-06,  5.67494e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(   1.0922e-05,  8.62802e-06,  6.22461e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.46531e-05,  1.10653e-05,  8.31806e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.59318e-05,  1.26858e-05,  9.23644e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.85843e-05,  1.54902e-05,   1.2506e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  2.42807e-05,  1.88826e-05,  1.35573e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  2.56286e-05,  1.93223e-05,  1.48395e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  2.44154e-05,  1.95696e-05,   1.6979e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  2.56003e-05,  1.93254e-05,  1.49079e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  2.46068e-05,  1.91539e-05,  1.38544e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(   1.8912e-05,  1.57287e-05,  1.24406e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.63737e-05,  1.29691e-05,  9.37932e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(   1.4723e-05,  1.09703e-05,  8.44797e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.11258e-05,  8.77951e-06,  6.29427e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  8.62237e-06,  7.31059e-06,  5.76465e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  7.15906e-06,  5.53673e-06,  3.95202e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  5.98322e-06,  3.99935e-06,  3.19531e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  4.16914e-06,  3.46227e-06,  2.22435e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  3.52191e-06,  2.76613e-06,  2.30197e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  3.16213e-06,  2.10136e-06,  1.61803e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  2.20634e-06,   1.6119e-06,  1.22854e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.70821e-06,  1.40671e-06,  9.71428e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-7.0) * scalingFactor;
    rgb += vec3(  1.43749e-06,  1.24737e-06,  1.03207e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  1.74169e-06,  1.28229e-06,  1.10133e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.10079e-06,  1.52175e-06,  1.24171e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.50911e-06,  1.95601e-06,  1.46037e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  3.18303e-06,  2.46278e-06,   1.5748e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  3.81602e-06,  3.04174e-06,  2.07389e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  4.74524e-06,  3.85411e-06,  2.63312e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  6.81251e-06,  4.46519e-06,  3.42275e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  8.21364e-06,  6.76201e-06,  4.35698e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  1.08919e-05,   8.5635e-06,  6.16486e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(   1.3211e-05,  1.15527e-05,  8.11411e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  1.99426e-05,  1.49275e-05,  1.08807e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.25984e-05,  1.91227e-05,  1.30134e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(   2.7644e-05,  2.33038e-05,   1.7482e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(   3.7772e-05,  2.91402e-05,  2.00443e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  3.71842e-05,  3.23825e-05,  2.38767e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  4.00253e-05,   3.4993e-05,  2.37181e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  3.70107e-05,  3.28534e-05,  2.39961e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  3.85047e-05,  2.96388e-05,  2.02798e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.80453e-05,  2.37334e-05,   1.7926e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.31947e-05,  1.89879e-05,  1.30654e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.07267e-05,  1.54675e-05,  1.09932e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  1.35213e-05,  1.18182e-05,  8.21854e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  1.11154e-05,  8.76905e-06,  6.28881e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  8.42472e-06,   6.8203e-06,  4.47422e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  7.02111e-06,  4.68607e-06,  3.43021e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  4.82826e-06,  3.87854e-06,  2.71263e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(   3.8641e-06,  3.10442e-06,  2.07635e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  3.38921e-06,  2.50242e-06,  1.57718e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.46565e-06,  1.99706e-06,  1.52996e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  2.09381e-06,  1.55233e-06,  1.23189e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-6.0) * scalingFactor;
    rgb += vec3(  1.80169e-06,  1.29954e-06,  1.18435e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.12276e-06,  1.25938e-06,   1.5489e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.37458e-06,  1.67302e-06,  1.57656e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.82257e-06,   2.1232e-06,  1.94311e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  4.16569e-06,  2.75054e-06,  1.95437e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  4.62855e-06,  3.55662e-06,  2.31658e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  6.12085e-06,  4.54578e-06,  2.91458e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  8.62249e-06,  5.40345e-06,  4.28468e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  1.04582e-05,  7.44556e-06,  5.59816e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  1.43449e-05,  1.08603e-05,  8.21599e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  1.98131e-05,  1.49057e-05,  1.07839e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(    2.906e-05,  2.04338e-05,  1.61056e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  3.70908e-05,  2.82293e-05,  2.10729e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  4.82789e-05,  3.71182e-05,  2.87084e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  6.83603e-05,  4.76055e-05,  3.52636e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  7.71231e-05,  5.79309e-05,  4.43727e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  8.16395e-05,  5.49123e-05,  4.89816e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  7.93854e-05,  5.85661e-05,  4.48483e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  6.86385e-05,  4.85817e-05,  3.56415e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  4.97963e-05,  3.73966e-05,  2.91318e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  3.79984e-05,  2.91662e-05,  2.14716e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.99978e-05,  2.07631e-05,  1.64867e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.07251e-05,  1.54599e-05,  1.09876e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  1.47365e-05,  1.09834e-05,  8.45954e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  1.08253e-05,  7.70702e-06,  5.66142e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  8.89182e-06,  5.53344e-06,  4.38262e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  6.27538e-06,  4.63694e-06,  2.95709e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  4.69124e-06,  3.60142e-06,  2.39916e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  4.10775e-06,    2.761e-06,  1.96055e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.98463e-06,  2.14834e-06,  1.83756e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.44375e-06,   1.7408e-06,  1.59172e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-5.0) * scalingFactor;
    rgb += vec3(  2.07213e-06,  1.31554e-06,  1.71032e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(   1.6123e-06,  1.58153e-06,  9.27998e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  2.00924e-06,  1.97491e-06,  1.06174e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  2.65499e-06,  2.34931e-06,  1.39667e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  3.77698e-06,  2.95792e-06,  1.67918e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  4.55217e-06,  3.84307e-06,  2.67827e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  6.03361e-06,  5.11881e-06,  3.47849e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  9.28595e-06,  6.28427e-06,  4.99026e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  1.14976e-05,  9.35624e-06,  6.23877e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  1.57401e-05,  1.26212e-05,  9.23584e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  2.24385e-05,  1.86549e-05,  1.27743e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  3.65429e-05,  2.78938e-05,  2.09045e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  4.92567e-05,  4.16283e-05,   2.8616e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  7.11659e-05,  6.12492e-05,  4.51899e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  0.000110878,  8.54653e-05,  5.95172e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  0.000129948,  0.000109402,  8.37135e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  0.000142168,  0.000111417,  8.57824e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  0.000131521,  0.000110934,  8.47909e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  0.000114375,  8.68194e-05,   6.0536e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  7.40817e-05,  6.36218e-05,  4.63111e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  5.03577e-05,  4.24558e-05,  2.93229e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(   3.7996e-05,  2.91682e-05,  2.14726e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  2.32101e-05,  1.90051e-05,  1.30775e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  1.63747e-05,  1.29677e-05,  9.38958e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  1.19177e-05,  9.58781e-06,  6.36152e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(   9.5375e-06,  6.33858e-06,  5.12335e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  6.24093e-06,  5.25016e-06,  3.48904e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  4.75909e-06,  3.97302e-06,   2.7546e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  3.78144e-06,  3.08995e-06,  1.75907e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  2.74059e-06,  2.35711e-06,  1.34013e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  2.12165e-06,  1.93446e-06,  1.04536e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-4.0) * scalingFactor;
    rgb += vec3(  1.61738e-06,  1.64377e-06,  1.04438e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  2.26788e-06,  2.11059e-06,  1.62388e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  3.03402e-06,  2.31405e-06,   1.7828e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(   3.5597e-06,  2.58926e-06,  1.86571e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  4.81178e-06,  3.29256e-06,  2.17136e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  5.86342e-06,  4.27488e-06,  3.00241e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  7.25435e-06,  5.59159e-06,  3.62988e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  1.06579e-05,  6.99411e-06,  5.47228e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  1.31335e-05,  1.04718e-05,  7.51081e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  1.83214e-05,  1.51603e-05,  1.21532e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  2.69904e-05,   2.2901e-05,  1.74348e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  4.67507e-05,  3.63431e-05,  2.84586e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(   7.0247e-05,  6.02677e-05,  4.45199e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(    0.0001192,  0.000101986,  8.17182e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  0.000211753,  0.000170242,  0.000125066) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  0.000290433,  0.000253724,  0.000199727) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  0.000395576,  0.000312056,  0.000243277) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  0.000303495,  0.000261139,  0.000203875) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  0.000221773,  0.000177749,   0.00012845) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  0.000123477,  0.000104943,  8.41033e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  7.41138e-05,  6.36434e-05,  4.63254e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  4.98071e-05,  3.74131e-05,  2.91427e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  2.80367e-05,  2.37344e-05,  1.79315e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  1.89038e-05,  1.57206e-05,  1.24167e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  1.32909e-05,  1.06381e-05,  7.69568e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  1.09302e-05,  7.19049e-06,  5.55694e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  7.48073e-06,  5.61938e-06,  3.75675e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  5.78126e-06,  4.22033e-06,  2.91343e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  5.23428e-06,  3.41002e-06,   2.2348e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  3.46404e-06,  2.74969e-06,  1.91017e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  2.96018e-06,  2.23213e-06,  1.69512e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-3.0) * scalingFactor;
    rgb += vec3(  2.50834e-06,  2.03817e-06,   1.8167e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  1.83391e-06,  1.12443e-06,  1.98727e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  2.27705e-06,  1.56478e-06,  2.17253e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  3.20067e-06,  2.30954e-06,  2.45848e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(   4.4587e-06,  3.09987e-06,  3.03812e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  5.76273e-06,  4.24012e-06,  3.75343e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  7.98541e-06,  6.07356e-06,  4.74006e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  1.19048e-05,  7.85905e-06,  6.54046e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  1.64157e-05,  1.23058e-05,  8.75621e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  2.35653e-05,  1.82487e-05,  1.33665e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(   3.5837e-05,  2.82726e-05,   1.9584e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  6.78822e-05,  4.67526e-05,  3.48807e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  0.000107926,  8.34889e-05,  5.87063e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  0.000206599,  0.000165864,  0.000122746) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  0.000512757,  0.000364265,  0.000252071) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(   0.00102414,  0.000791414,  0.000564745) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(   0.00134307,   0.00106991,  0.000792053) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(   0.00108307,  0.000841332,  0.000588734) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  0.000550801,  0.000383963,  0.000266286) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  0.000221794,  0.000177777,  0.000128476) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  0.000114358,  8.67929e-05,  6.05237e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  6.86029e-05,  4.85474e-05,  3.56009e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  3.84784e-05,  2.95915e-05,   2.0242e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  2.45802e-05,  1.91425e-05,  1.38446e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  1.67213e-05,  1.26552e-05,  8.85352e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  1.27837e-05,  8.07008e-06,   6.7303e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  7.87333e-06,  6.25403e-06,  4.72363e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  6.03303e-06,  4.45623e-06,  3.97376e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  4.63682e-06,  3.11601e-06,  2.91111e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(   3.2243e-06,   2.3341e-06,  2.70638e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  2.53569e-06,   1.6877e-06,  1.99804e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-2.0) * scalingFactor;
    rgb += vec3(  1.85996e-06,  1.13349e-06,  2.19194e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  2.02264e-06,  1.73736e-06,  1.01734e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(   2.9141e-06,  1.94242e-06,  1.15999e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  3.35246e-06,  2.44971e-06,  1.41613e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  5.06663e-06,  3.59017e-06,  1.92935e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  5.91372e-06,  4.64844e-06,  2.68203e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  7.34117e-06,  5.98901e-06,  3.92315e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  1.30282e-05,  8.19897e-06,  6.01772e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  1.47358e-05,  1.30135e-05,  8.82847e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  2.45471e-05,  1.99464e-05,   1.4615e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  3.88554e-05,  3.06499e-05,  2.30271e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  6.92781e-05,  5.49261e-05,  4.30713e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  0.000129174,   0.00010839,  7.94723e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  0.000270265,  0.000242088,  0.000195603) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  0.000962815,  0.000738053,  0.000540006) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(   0.00437616,   0.00373232,   0.00278031) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(    0.0184737,     0.012691,    0.0104018) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(   0.00525292,   0.00437237,   0.00314621) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(   0.00108306,   0.00084129,  0.000588707) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  0.000303467,  0.000261105,   0.00020382) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  0.000131513,  0.000110933,  8.47854e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  7.94091e-05,  5.85883e-05,  4.48716e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  3.70397e-05,  3.28805e-05,  2.40141e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  2.56226e-05,  1.93369e-05,  1.49268e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  1.73594e-05,  1.35281e-05,  9.17641e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  1.17757e-05,   8.4448e-06,  6.19594e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  8.74553e-06,  6.76016e-06,   3.9845e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  5.65281e-06,   4.4824e-06,  2.86041e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  5.39649e-06,  3.48146e-06,  1.88301e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  3.28424e-06,  2.78926e-06,  1.57083e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  2.66588e-06,   1.9367e-06,  1.07929e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (-1.0) * scalingFactor;
    rgb += vec3(  2.38795e-06,  1.61359e-06,  1.12613e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  2.35285e-06,  2.09606e-06,  2.18655e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  3.20816e-06,  2.21575e-06,  2.36498e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  3.35096e-06,   2.9143e-06,  2.56818e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  5.45144e-06,  3.94212e-06,  3.21468e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  5.70309e-06,  4.50704e-06,  3.78157e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  7.96426e-06,  6.29859e-06,  5.24039e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  1.26997e-05,  8.05143e-06,  6.79019e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  1.64173e-05,  1.35446e-05,  1.02005e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(   2.7549e-05,  1.84135e-05,  1.48706e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(   3.6878e-05,  3.02604e-05,   2.5349e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  8.26209e-05,  5.92056e-05,  4.25105e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  0.000130591,  0.000104839,  8.93133e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  0.000304199,   0.00025287,  0.000205534) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(    0.0013116,   0.00102573,  0.000753075) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(    0.0103191,    0.0103248,   0.00744039) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(      0.89269,     0.913544,     0.934333) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(    0.0184735,    0.0126908,    0.0104018) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(   0.00134307,   0.00106994,  0.000791983) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(   0.00039555,  0.000312029,  0.000243292) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  0.000142166,  0.000111431,  8.57708e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(   8.1642e-05,  5.48936e-05,  4.89733e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  4.00097e-05,   3.4998e-05,  2.37234e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  2.44223e-05,  1.95671e-05,  1.69695e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  1.91594e-05,  1.22634e-05,  9.42952e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  1.20395e-05,  8.52254e-06,  7.67841e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  9.23193e-06,  7.04258e-06,  4.82277e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(   6.1897e-06,   4.8069e-06,  4.25281e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  5.20878e-06,  3.53234e-06,  2.92919e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  3.63283e-06,  3.19237e-06,  2.87435e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  2.76507e-06,  2.39372e-06,   2.1367e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 0.0) * scalingFactor;
    rgb += vec3(  2.66117e-06,   1.8504e-06,  2.42324e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  1.95869e-06,  1.63014e-06,  9.02907e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  2.85932e-06,  1.88377e-06,   1.0395e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  3.37086e-06,  2.35576e-06,  1.31025e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  4.93177e-06,  3.47451e-06,  1.80339e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  5.91882e-06,  4.60308e-06,  2.61318e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  7.26513e-06,   5.9171e-06,  3.82572e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(     1.29e-05,   8.0812e-06,  5.96623e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  1.47162e-05,  1.27161e-05,  8.73757e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  2.39142e-05,  1.98298e-05,  1.45345e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  3.86882e-05,  3.05909e-05,  2.28725e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  6.80564e-05,  5.42397e-05,  4.29265e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  0.000126353,  0.000106879,  7.84764e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  0.000264571,  0.000238355,   0.00019216) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  0.000892573,  0.000700233,  0.000521069) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(   0.00358465,   0.00334392,   0.00253027) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(    0.0103192,    0.0103248,   0.00744042) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(   0.00437621,   0.00373239,   0.00278035) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(   0.00102404,  0.000791375,  0.000564754) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  0.000290495,  0.000253789,  0.000199768) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  0.000129926,  0.000109354,  8.36813e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  7.70661e-05,  5.78959e-05,  4.43495e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  3.71642e-05,  3.23611e-05,  2.38472e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  2.56111e-05,  1.93001e-05,    1.482e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  1.69588e-05,  1.33933e-05,  9.07203e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  1.17454e-05,  8.26794e-06,   6.0804e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  8.54672e-06,  6.57451e-06,  3.89947e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  5.53885e-06,  4.39717e-06,  2.74304e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  5.34994e-06,  3.41827e-06,  1.76825e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  3.23152e-06,  2.66843e-06,  1.43381e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  2.65124e-06,  1.85226e-06,   9.6789e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 1.0) * scalingFactor;
    rgb += vec3(  2.32947e-06,  1.51794e-06,  1.00098e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  1.86933e-06,  1.25157e-06,  2.10808e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  2.26345e-06,  1.68801e-06,   2.2899e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  3.15649e-06,  2.44933e-06,  2.54734e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  4.54371e-06,  3.25033e-06,  3.11323e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  5.69154e-06,  4.36271e-06,  3.78144e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  8.03762e-06,  6.20556e-06,  4.72128e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  1.20362e-05,  7.99307e-06,  6.44383e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  1.62764e-05,  1.24167e-05,  8.63396e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  2.34382e-05,  1.80626e-05,  1.32758e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  3.52763e-05,  2.77407e-05,  1.93455e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(   6.6344e-05,  4.56776e-05,   3.4476e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  0.000104723,  8.18185e-05,  5.72271e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  0.000200211,   0.00015914,  0.000120303) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(   0.00047393,  0.000349131,   0.00024088) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  0.000892585,  0.000700259,  0.000521102) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(   0.00131156,   0.00102574,  0.000753141) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  0.000962853,  0.000738028,  0.000539965) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  0.000512794,  0.000364297,  0.000252081) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  0.000211677,  0.000170147,  0.000124968) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  0.000110918,  8.55269e-05,  5.95822e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  6.84271e-05,  4.76446e-05,  3.52854e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(   3.7774e-05,   2.9135e-05,  2.00385e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  2.42685e-05,  1.88774e-05,  1.35617e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  1.66596e-05,   1.2695e-05,  8.76328e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  1.27299e-05,  8.17776e-06,  6.65477e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  7.99829e-06,   6.4244e-06,  4.69245e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  6.05399e-06,  4.54037e-06,   3.9765e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  4.58612e-06,  3.22745e-06,  2.97252e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  3.25102e-06,  2.48089e-06,   2.7949e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  2.47956e-06,  1.82004e-06,  2.10274e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 2.0) * scalingFactor;
    rgb += vec3(  1.83295e-06,  1.22692e-06,  2.32898e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  2.31004e-06,  1.96247e-06,  1.55774e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  3.04491e-06,  2.16122e-06,  1.70109e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  3.60637e-06,  2.45322e-06,  1.79057e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  4.76391e-06,  3.14057e-06,  2.09795e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  5.74675e-06,  4.16201e-06,  2.90683e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  7.08844e-06,   5.5221e-06,  3.58754e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  1.03325e-05,  6.84944e-06,  5.52022e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(   1.2914e-05,  1.03462e-05,  7.60689e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  1.81381e-05,  1.49584e-05,   1.2145e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  2.66104e-05,  2.25516e-05,  1.69766e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(   4.5656e-05,  3.58899e-05,  2.80601e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(   6.7943e-05,   5.8311e-05,  4.38858e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(   0.00011165,  9.88777e-05,  7.84991e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  0.000200185,  0.000159116,  0.000120293) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  0.000264597,  0.000238394,   0.00019222) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  0.000304208,  0.000252867,  0.000205499) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  0.000270238,  0.000242057,  0.000195582) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  0.000206645,  0.000165942,  0.000122823) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  0.000119196,  0.000101998,  8.17403e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  7.10987e-05,  6.11625e-05,  4.50978e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  4.82556e-05,  3.71359e-05,  2.87172e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  2.76877e-05,   2.3337e-05,  1.75134e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  1.86049e-05,  1.55037e-05,  1.25116e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  1.30104e-05,  1.05363e-05,  7.82667e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  1.07002e-05,  7.15233e-06,  5.59269e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  7.25011e-06,  5.50299e-06,   3.7337e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  5.68337e-06,  4.14248e-06,  2.85758e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  5.17903e-06,  3.26701e-06,  2.16952e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  3.47279e-06,  2.60006e-06,  1.83143e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  3.03137e-06,  2.06696e-06,  1.62567e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 3.0) * scalingFactor;
    rgb += vec3(  2.56915e-06,  1.90139e-06,  1.73689e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  1.54159e-06,  1.57849e-06,  8.96327e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  1.99154e-06,  1.96148e-06,  1.07605e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  2.64888e-06,  2.32658e-06,  1.43223e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  3.84243e-06,  2.92535e-06,  1.76659e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(   4.6277e-06,  3.78615e-06,  2.82868e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(   6.0384e-06,  5.09318e-06,  3.59989e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  9.06877e-06,  6.25924e-06,  4.94438e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  1.12739e-05,  9.08886e-06,  6.06095e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  1.54408e-05,  1.24301e-05,   9.0506e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  2.20323e-05,  1.85915e-05,  1.27181e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  3.51726e-05,  2.72001e-05,  2.03864e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  4.87925e-05,  4.04486e-05,  2.80137e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  6.79272e-05,  5.83017e-05,  4.38734e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  0.000104736,  8.18381e-05,  5.72344e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  0.000126357,  0.000106893,  7.84912e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  0.000130573,  0.000104814,  8.93192e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  0.000129237,  0.000108451,  7.95082e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  0.000107854,  8.34077e-05,  5.86354e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  7.03212e-05,  6.03611e-05,  4.45972e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  4.92689e-05,  4.16369e-05,  2.86284e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  3.70369e-05,  2.81531e-05,  2.10049e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  2.25684e-05,  1.91029e-05,  1.30012e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  1.59492e-05,  1.27053e-05,  9.25248e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  1.17554e-05,   9.3931e-06,  6.15665e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  9.55018e-06,  6.26965e-06,  5.08079e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  6.23328e-06,  5.25703e-06,  3.61333e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  4.79904e-06,   3.9406e-06,   2.9198e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  3.82492e-06,  3.05843e-06,  1.85748e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  2.74327e-06,   2.3277e-06,  1.38152e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(  2.11041e-06,  1.92929e-06,  1.06871e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 4.0) * scalingFactor;
    rgb += vec3(   1.5504e-06,  1.65815e-06,  1.02208e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  2.06401e-06,  1.26021e-06,  1.53421e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(   2.3075e-06,  1.70141e-06,  1.52433e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  2.76492e-06,  2.16181e-06,  1.87589e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  4.04882e-06,  2.79501e-06,  1.87089e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  4.61173e-06,  3.55365e-06,  2.24549e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  6.03269e-06,  4.42702e-06,  2.88422e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  8.47397e-06,   5.1663e-06,  4.21803e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  1.03552e-05,  7.42571e-06,  5.56772e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(    1.428e-05,  1.08004e-05,  8.08171e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  1.88922e-05,  1.45508e-05,  1.06621e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  2.88095e-05,  1.98768e-05,  1.58539e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  3.51667e-05,  2.71997e-05,  2.03875e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  4.56383e-05,   3.5876e-05,  2.80504e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(    6.638e-05,  4.57041e-05,  3.45147e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  6.80499e-05,  5.42279e-05,  4.29061e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  8.26151e-05,  5.92139e-05,  4.25129e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  6.93185e-05,  5.49625e-05,  4.30996e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  6.78372e-05,  4.67258e-05,  3.48582e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  4.67446e-05,   3.6314e-05,  2.84428e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  3.66092e-05,  2.79606e-05,  2.09681e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  2.90709e-05,  2.04466e-05,  1.61133e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  1.98909e-05,  1.48771e-05,   1.0839e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  1.46202e-05,  1.10409e-05,  8.29309e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  1.06874e-05,  7.57997e-06,  5.64979e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  8.68131e-06,  5.35866e-06,  4.34339e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(   6.1933e-06,  4.50403e-06,  2.95564e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  4.69528e-06,  3.60322e-06,  2.31285e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  4.15206e-06,  2.83397e-06,  1.86923e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  2.92715e-06,  2.19736e-06,  1.77342e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  2.34054e-06,  1.77728e-06,  1.53572e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 5.0) * scalingFactor;
    rgb += vec3(  2.00132e-06,  1.33225e-06,  1.69004e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  1.72747e-06,  1.36121e-06,  1.10915e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.07618e-06,  1.58198e-06,  1.25237e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.42489e-06,  1.94533e-06,  1.45822e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   3.1469e-06,  2.36707e-06,  1.53986e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  3.73148e-06,  2.92673e-06,  2.08351e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  4.63976e-06,  3.81504e-06,  2.59563e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   6.7645e-06,  4.52317e-06,  3.36449e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  8.11662e-06,  6.62377e-06,  4.34181e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  1.05107e-05,  8.48028e-06,  6.14372e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  1.32421e-05,  1.11999e-05,  7.97727e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  1.88965e-05,  1.45598e-05,  1.06687e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.20191e-05,  1.85763e-05,  1.27068e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.66165e-05,  2.25477e-05,  1.69682e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  3.53079e-05,  2.77875e-05,  1.93836e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   3.8653e-05,  3.05656e-05,  2.28539e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   3.6885e-05,  3.02576e-05,  2.53436e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  3.88784e-05,  3.06698e-05,  2.30569e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  3.58428e-05,  2.82831e-05,  1.95901e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.69439e-05,  2.28681e-05,  1.74071e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.24488e-05,  1.86598e-05,  1.27801e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  1.98685e-05,   1.4952e-05,  1.08285e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  1.32151e-05,  1.15585e-05,   8.1166e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   1.0891e-05,  8.59396e-06,   6.1947e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   8.2861e-06,  6.73034e-06,   4.4436e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  6.93213e-06,    4.654e-06,  3.37448e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  4.79302e-06,  3.90275e-06,  2.64056e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   3.8007e-06,  2.99249e-06,  2.10554e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(   3.2609e-06,  2.40402e-06,  1.55232e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.39413e-06,   1.9946e-06,  1.53152e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  2.09076e-06,  1.61622e-06,  1.23935e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 6.0) * scalingFactor;
    rgb += vec3(  1.80093e-06,  1.35941e-06,  1.19226e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.38699e-06,  1.16897e-06,  1.01257e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.62919e-06,   1.2989e-06,   1.0227e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  2.14007e-06,  1.56129e-06,  1.26111e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  3.09323e-06,  2.13072e-06,  1.71172e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  3.37443e-06,  2.78172e-06,  2.18382e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  4.05258e-06,  3.22341e-06,    2.081e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  5.77653e-06,  3.96977e-06,  3.17585e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  7.00842e-06,  5.28267e-06,  3.82461e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  7.91852e-06,  6.97677e-06,  5.55685e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.05215e-05,  8.48838e-06,  6.14704e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.42673e-05,  1.07883e-05,   8.0722e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.54397e-05,  1.24294e-05,  9.03885e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.81463e-05,  1.49642e-05,  1.21685e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  2.34661e-05,   1.8076e-05,   1.3288e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  2.38962e-05,  1.98187e-05,  1.45182e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  2.75389e-05,  1.84108e-05,  1.48763e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  2.45641e-05,  1.99704e-05,   1.4637e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  2.35813e-05,  1.82532e-05,  1.33632e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.82998e-05,  1.51511e-05,  1.21498e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.57204e-05,  1.26007e-05,  9.21405e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(   1.4367e-05,  1.08836e-05,  8.23444e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.09275e-05,  8.59738e-06,  6.19896e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  8.27164e-06,   7.1382e-06,  5.67413e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  6.99508e-06,  5.42101e-06,   3.8839e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  5.96154e-06,  3.98463e-06,  3.21601e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  4.09773e-06,  3.31623e-06,  2.12388e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  3.39572e-06,  2.79866e-06,   2.2115e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  3.17795e-06,  2.18695e-06,  1.72694e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  2.24834e-06,   1.5856e-06,  1.29812e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.68629e-06,  1.30553e-06,  1.01691e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 7.0) * scalingFactor;
    rgb += vec3(  1.41987e-06,  1.18069e-06,  1.04601e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.31742e-06,  1.08977e-06,  1.04512e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.76169e-06,   1.3771e-06,  1.03666e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  2.09274e-06,  1.66349e-06,  1.19359e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  2.57539e-06,  1.84267e-06,  1.22019e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  2.82378e-06,   2.2097e-06,   1.4329e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  3.71541e-06,  2.91082e-06,  1.88593e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  5.01766e-06,  3.27095e-06,  2.63473e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  5.38693e-06,  4.61137e-06,   3.2466e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  7.01719e-06,  5.28995e-06,  3.82409e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  8.11036e-06,  6.61778e-06,  4.34129e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.03485e-05,   7.4223e-06,  5.56169e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.12709e-05,  9.08019e-06,  6.05894e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.29344e-05,  1.03758e-05,   7.6231e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.62754e-05,  1.24176e-05,  8.64336e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.47078e-05,  1.27014e-05,   8.7304e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.64195e-05,  1.35458e-05,  1.01975e-05) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.47527e-05,  1.30382e-05,  8.84279e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.64165e-05,  1.22995e-05,  8.75705e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.31354e-05,  1.04776e-05,  7.51399e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.14831e-05,  9.33907e-06,  6.22163e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.04505e-05,  7.43774e-06,  5.59639e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  8.23834e-06,  6.77991e-06,  4.37017e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  7.09041e-06,   5.3673e-06,  3.91434e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  5.61599e-06,  4.64977e-06,    3.252e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  5.08163e-06,  3.33349e-06,  2.73115e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  3.78814e-06,  2.91035e-06,  1.87437e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  2.87609e-06,  2.29745e-06,  1.40649e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  2.57617e-06,  1.81836e-06,  1.27774e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  2.12249e-06,  1.68772e-06,   1.1907e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.82428e-06,  1.40728e-06,   1.0921e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 8.0) * scalingFactor;
    rgb += vec3(  1.34097e-06,  1.09701e-06,  1.06372e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.52366e-06,  7.97804e-07,   9.0729e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(   1.6628e-06,  1.06427e-06,  8.45857e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(   1.8071e-06,    1.314e-06,  1.19963e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  2.51581e-06,  1.51583e-06,  1.05099e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  2.91262e-06,  1.78692e-06,  1.43055e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  3.35294e-06,  2.42941e-06,  1.89847e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  4.68527e-06,  2.76245e-06,  2.48429e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  5.02521e-06,  3.27484e-06,  2.63328e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  5.77485e-06,  3.96706e-06,  3.17438e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(   6.7569e-06,  4.51508e-06,  3.36456e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  8.47416e-06,  5.16041e-06,  4.19996e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  9.06326e-06,  6.26969e-06,  4.96602e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.03675e-05,  6.86007e-06,  5.52987e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.20278e-05,  7.99352e-06,   6.4486e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.28846e-05,  8.07189e-06,   5.9593e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.26983e-05,   8.0546e-06,  6.79392e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.30486e-05,  8.20858e-06,  6.02404e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.19017e-05,  7.85992e-06,  6.55072e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.06713e-05,  6.99318e-06,  5.46645e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  9.27105e-06,  6.28635e-06,  4.99386e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  8.60715e-06,   5.3928e-06,   4.2705e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  6.81487e-06,  4.46494e-06,  3.42989e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  5.80371e-06,  4.01993e-06,  3.09804e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(   5.1155e-06,  3.35422e-06,  2.70142e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  4.76646e-06,  2.80174e-06,  2.49178e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  3.39942e-06,  2.47775e-06,   1.9308e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  2.99526e-06,   1.8187e-06,  1.51532e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  2.57325e-06,  1.52102e-06,  9.70933e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.81154e-06,  1.29554e-06,  1.21394e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.67595e-06,  1.14689e-06,  9.03005e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + ( 9.0) * scalingFactor;
    rgb += vec3(  1.56836e-06,  8.23231e-07,  9.33471e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  1.00896e-06,  1.00928e-06,  8.05663e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  1.17287e-06,  1.00719e-06,  8.12727e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  1.59282e-06,  1.12374e-06,  7.53177e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  2.10821e-06,  1.41112e-06,  1.01995e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  2.12268e-06,  2.00378e-06,  1.39721e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  2.94308e-06,  2.24061e-06,  1.55852e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(   3.3592e-06,   2.4315e-06,  1.89726e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  3.70998e-06,  2.91412e-06,  1.88701e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  4.04893e-06,  3.22088e-06,  2.08238e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  4.63623e-06,  3.81104e-06,  2.58332e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  6.02875e-06,  4.42039e-06,  2.89454e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  6.05652e-06,   5.1098e-06,  3.60231e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  7.09319e-06,  5.53223e-06,  3.59839e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  8.03694e-06,  6.20067e-06,   4.7206e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  7.25516e-06,  5.91258e-06,  3.82439e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  7.96697e-06,  6.29961e-06,   5.2408e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  7.34996e-06,  5.99336e-06,  3.93128e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  7.99228e-06,  6.08364e-06,  4.75012e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  7.25492e-06,  5.58942e-06,  3.62046e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  6.03373e-06,  5.12567e-06,  3.48386e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(   6.1093e-06,  4.52866e-06,  2.91231e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  4.73821e-06,   3.8494e-06,  2.62838e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  4.13008e-06,  3.34917e-06,  2.21585e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  3.71346e-06,  2.93853e-06,  1.85494e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  3.39633e-06,  2.41779e-06,  1.90811e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  2.97282e-06,  2.28189e-06,  1.60389e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  2.22438e-06,  1.96871e-06,  1.37219e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  2.09785e-06,  1.52115e-06,  1.08219e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  1.62819e-06,  1.12056e-06,  7.21399e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  1.21739e-06,  9.43454e-07,  8.27875e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (10.0) * scalingFactor;
    rgb += vec3(  9.86553e-07,  1.03685e-06,  8.39939e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.05824e-06,  7.82533e-07,  8.02066e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.22549e-06,  7.31426e-07,  6.90941e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.37109e-06,  1.05684e-06,   9.1985e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.87225e-06,  1.52585e-06,  1.13953e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  2.12491e-06,  1.61293e-06,  1.42143e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  2.12592e-06,  2.00237e-06,  1.39497e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(   2.9103e-06,  1.78658e-06,  1.43139e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  2.82294e-06,  2.20568e-06,   1.4359e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  3.36915e-06,  2.78271e-06,  2.17788e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  3.73093e-06,  2.91476e-06,  2.07471e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(    4.613e-06,  3.56809e-06,  2.26029e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  4.64007e-06,  3.78874e-06,  2.82813e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  5.75256e-06,  4.17299e-06,  2.92057e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  5.68942e-06,  4.35791e-06,  3.77915e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  5.91024e-06,  4.59992e-06,  2.60958e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  5.70319e-06,  4.50934e-06,   3.7792e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(   5.9181e-06,  4.65219e-06,  2.68994e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  5.77429e-06,  4.24685e-06,  3.75966e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  5.86419e-06,  4.27352e-06,  2.99749e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  4.54886e-06,   3.8437e-06,  2.68118e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  4.62719e-06,  3.55655e-06,  2.31722e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  3.80728e-06,  3.02972e-06,  2.07319e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  3.44162e-06,  2.76409e-06,  2.25709e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  2.88059e-06,  2.21379e-06,  1.43956e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  2.93894e-06,   1.7955e-06,  1.41761e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  2.18374e-06,  1.99193e-06,  1.42745e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  2.15083e-06,  1.71587e-06,  1.42912e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.92546e-06,  1.49785e-06,  1.14683e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.35151e-06,  1.12463e-06,  9.69603e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.21107e-06,  7.54048e-07,  6.04977e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (11.0) * scalingFactor;
    rgb += vec3(  1.09064e-06,  7.42002e-07,  8.97329e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  9.23316e-07,  6.13861e-07,  5.29891e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.18268e-06,  9.42333e-07,  6.59794e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.46065e-06,   1.1288e-06,  9.54119e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.86861e-06,  1.22218e-06,  1.12051e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.87695e-06,  1.52483e-06,  1.13473e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(   2.1086e-06,  1.41059e-06,  1.02006e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  2.51313e-06,  1.51327e-06,  1.05469e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(   2.5749e-06,  1.84626e-06,  1.22066e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  3.08869e-06,  2.11912e-06,  1.70101e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  3.13811e-06,  2.37038e-06,  1.54787e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  4.06419e-06,  2.80353e-06,  1.87498e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(   3.8447e-06,  2.92963e-06,  1.76758e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  4.77047e-06,  3.14861e-06,  2.10786e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  4.53874e-06,  3.24402e-06,   3.1113e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  4.92667e-06,  3.47295e-06,  1.80087e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(   5.4549e-06,  3.94153e-06,  3.21267e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  5.06909e-06,  3.59648e-06,  1.93929e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  4.46919e-06,  3.10282e-06,  3.04084e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  4.80708e-06,  3.29773e-06,  2.16944e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  3.77768e-06,  2.95302e-06,  1.67875e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  4.17321e-06,   2.7505e-06,  1.96159e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  3.16973e-06,  2.46196e-06,  1.57083e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  3.14692e-06,  2.02386e-06,   1.6071e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(    2.657e-06,  1.92106e-06,  1.32214e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  2.46103e-06,  1.59459e-06,  1.06572e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  2.19707e-06,  1.38161e-06,  9.95055e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.85693e-06,  1.52764e-06,   1.1173e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.91489e-06,  1.25458e-06,  1.11885e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.51891e-06,  1.11699e-06,   9.6868e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  1.18288e-06,  9.62114e-07,  6.77157e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (12.0) * scalingFactor;
    rgb += vec3(  9.54475e-07,   6.6942e-07,  5.81071e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  8.96675e-07,   7.0891e-07,  5.42991e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.10126e-06,  8.84072e-07,  1.08729e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.13354e-06,  1.03329e-06,  9.84172e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.46034e-06,  1.12482e-06,  9.50966e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.37003e-06,  1.05901e-06,  9.22589e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.59574e-06,  1.12287e-06,  7.52057e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.80581e-06,  1.31698e-06,  1.20455e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.09385e-06,  1.65703e-06,  1.18178e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.12839e-06,  1.55511e-06,  1.26266e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.43604e-06,  1.95853e-06,  1.46438e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(   2.7689e-06,  2.16126e-06,  1.87755e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.65522e-06,  2.33408e-06,  1.43981e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(   3.6089e-06,  2.45689e-06,  1.79666e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  3.15087e-06,  2.44614e-06,  2.54611e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  3.36965e-06,  2.35515e-06,  1.31121e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(   3.3526e-06,  2.91423e-06,  2.56431e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  3.35376e-06,  2.45387e-06,  1.42363e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  3.20718e-06,  2.31201e-06,  2.45556e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  3.56198e-06,  2.59653e-06,  1.87052e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.65488e-06,   2.3413e-06,  1.39572e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.82234e-06,  2.12892e-06,  1.94708e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.50694e-06,  1.95603e-06,  1.46019e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.08312e-06,  1.60035e-06,  1.19831e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  2.14425e-06,  1.69621e-06,  1.24663e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.88781e-06,   1.2992e-06,  1.15017e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.54017e-06,  1.15726e-06,  7.70731e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.41722e-06,  1.03867e-06,  9.55892e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.48093e-06,  1.13415e-06,  9.33709e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(   1.1161e-06,  1.04387e-06,  9.72962e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  1.14928e-06,  8.79967e-07,  1.04242e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (13.0) * scalingFactor;
    rgb += vec3(  8.86778e-07,    7.109e-07,  6.57165e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  8.52429e-07,  8.11928e-07,  8.86009e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  9.47832e-07,  8.29921e-07,  5.65097e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.09937e-06,  8.81277e-07,  1.08822e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.17974e-06,   9.4708e-07,  6.61366e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.22753e-06,  7.26926e-07,  6.91543e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.16808e-06,   1.0108e-06,   8.1636e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.66758e-06,  1.06113e-06,  8.38981e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.74993e-06,  1.36679e-06,  1.03521e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.63074e-06,  1.30517e-06,  1.03114e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.08866e-06,  1.58711e-06,  1.25339e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.30593e-06,  1.70274e-06,  1.52622e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.99877e-06,  1.97193e-06,  1.08143e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  3.04823e-06,  2.15965e-06,   1.7071e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.25788e-06,  1.68582e-06,  2.28767e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.85614e-06,  1.88247e-06,  1.04155e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  3.20922e-06,  2.21399e-06,  2.36088e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.91697e-06,  1.95067e-06,  1.16448e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.28164e-06,  1.56331e-06,  2.16925e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  3.03854e-06,  2.32264e-06,  1.78807e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.00341e-06,  1.96905e-06,  1.06084e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.37665e-06,  1.67429e-06,   1.5779e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  2.09982e-06,  1.52356e-06,  1.24708e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.64929e-06,  1.40929e-06,  9.82345e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.69337e-06,  1.31393e-06,  1.01865e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.75293e-06,  1.03612e-06,  8.56839e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.13625e-06,  1.03785e-06,  8.17597e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.26554e-06,   7.9499e-07,  6.95249e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  1.16116e-06,  9.38633e-07,  6.90197e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(   1.1239e-06,  8.84296e-07,  1.05233e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  9.76236e-07,  8.33707e-07,   6.1838e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (14.0) * scalingFactor;
    rgb += vec3(  8.68196e-07,  8.11591e-07,  9.06209e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-15.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  7.85044e-07,  6.34061e-07,  1.10681e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-14.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  8.50185e-07,  8.07645e-07,  8.85885e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-13.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  8.99389e-07,  7.10284e-07,  5.41903e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-12.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  9.24169e-07,  6.12856e-07,  5.35276e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-11.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.05474e-06,  7.89788e-07,  8.07169e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-10.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.01225e-06,  1.00809e-06,  8.03834e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-9.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.51792e-06,   7.9174e-07,  8.96787e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-8.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.31046e-06,  1.08799e-06,  1.05206e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-7.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.39554e-06,  1.17551e-06,  1.01733e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-6.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.73181e-06,  1.36246e-06,  1.11076e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-5.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  2.06726e-06,  1.26192e-06,  1.53742e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-4.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.54417e-06,  1.58919e-06,  8.98799e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-3.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(   2.3109e-06,  1.96156e-06,  1.56198e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-2.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.86819e-06,  1.24812e-06,  2.10722e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (-1.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.95619e-06,  1.62967e-06,  9.06136e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 0.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  2.35201e-06,  2.09448e-06,  2.18205e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 1.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  2.02571e-06,  1.74399e-06,  1.02212e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 2.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.83812e-06,  1.12204e-06,  1.98279e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 3.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  2.27095e-06,  2.11717e-06,  1.63034e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 4.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.61344e-06,  1.57931e-06,  9.29401e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 5.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  2.11837e-06,  1.25876e-06,  1.55215e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 6.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.74261e-06,   1.2847e-06,   1.1053e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 7.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.40983e-06,  1.23878e-06,  1.00164e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 8.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.27235e-06,  9.95884e-07,   1.0084e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + ( 9.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.47989e-06,   8.5385e-07,  8.74438e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (10.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.11066e-06,  1.02648e-06,   8.4288e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (11.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  1.00377e-06,  7.71175e-07,   8.2256e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (12.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  9.80762e-07,  6.57421e-07,  5.76427e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (13.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  9.02548e-07,  6.96246e-07,  5.11302e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (14.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  8.45102e-07,  8.26273e-07,  9.33155e-07) * texture2DRect(se_rgbaTex, pos).xyz;
    pos.x = uvPos.x + (15.0) * scalingFactor;
    pos.y = uvPos.y + (15.0) * scalingFactor;
    rgb += vec3(  7.65669e-07,  6.28014e-07,  1.10875e-06) * texture2DRect(se_rgbaTex, pos).xyz;
    
    gl_FragColor = vec4(rgb.x, rgb.y, rgb.z, rgba.w);
    return;
}